# -.- coding: utf-8 -.-

# Zeitgeist
#
# Copyright © 2009-2010 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
# Copyright © 2009 Mikkel Kamstrup Erlandsen <mikkel.kamstrup@gmail.com>
# Copyright © 2009 Markus Korn <thekorn@gmx.de>
# Copyright © 2010 Michal Hruby <michal.mhr@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import fnmatch

from zeitgeist.datamodel import Interpretation

class SimpleMatch(object):
    """ Wrapper around fnmatch.fnmatch which allows to define mimetype
    patterns by using shell-style wildcards.
    """

    def __init__(self, pattern):
        self.__pattern = pattern

    def match(self, text):
        return fnmatch.fnmatch(text, self.__pattern)

    def __repr__(self):
        return "%s(%r)" %(self.__class__.__name__, self.__pattern)

class MimeTypeSet(set):
    """ Set which allows to match against a string or an object with a
    match() method.
    """

    def __init__(self, *items):
        super(MimeTypeSet, self).__init__()
        self.__pattern = set()
        for item in items:
            if isinstance(item, (str, unicode)):
                self.add(item)
            elif hasattr(item, "match"):
                self.__pattern.add(item)
            else:
                raise ValueError("Bad mimetype '%s'" %item)

    def __contains__(self, mimetype):
        result = super(MimeTypeSet, self).__contains__(mimetype)
        if not result:
            for pattern in self.__pattern:
                if pattern.match(mimetype):
                    return True
        return result

    def __len__(self):
        return super(MimeTypeSet, self).__len__() + len(self.__pattern)

    def __repr__(self):
        items = ", ".join(sorted(map(repr, self | self.__pattern)))
        return "%s(%s)" %(self.__class__.__name__, items)

DOCUMENT_MIMETYPES = [
        # Covers:
        #    vnd.corel-draw
        #    vnd.ms-powerpoint
        #    vnd.ms-excel
        #    vnd.oasis.opendocument.*
        #    vnd.stardivision.*
        #    vnd.sun.xml.*
        #SimpleMatch(u"application/vnd.*"),
        # Covers: x-applix-word, x-applix-spreadsheet, x-applix-presents
        SimpleMatch(u"application/x-applix-*"),
        # Covers: x-kword, x-kspread, x-kpresenter, x-killustrator
        re.compile(u"application/x-k(word|spread|presenter|illustrator)"),
        u"application/ms-powerpoint",
        u"application/msword",
        u"application/pdf",
        u"application/postscript",
        u"application/ps",
        u"application/rtf",
        u"application/x-abiword",
        u"application/x-gnucash",
        u"application/x-gnumeric",
        SimpleMatch(u"application/x-java*"),
        SimpleMatch(u"*/x-tex"),
        SimpleMatch(u"*/x-latex"),
        SimpleMatch(u"*/x-dvi"),
        u"text/plain"
]

IMAGE_MIMETYPES = [
        # Covers:
        #    vnd.corel-draw
        #u"application/vnd.corel-draw",
        # Covers: x-kword, x-kspread, x-kpresenter, x-killustrator
        re.compile(u"application/x-k(word|spread|presenter|illustrator)"),
        #SimpleMatch(u"image/*"),
]

AUDIO_MIMETYPES = [
        SimpleMatch(u"audio/*"),
        u"application/ogg"
]

VIDEO_MIMETYPES = [
        SimpleMatch(u"video/*"),
        u"application/ogg"
]

DEVELOPMENT_MIMETYPES = [
        u"application/ecmascript",
        u"application/javascript",
        u"application/x-csh",
        u"application/x-designer",
        u"application/x-desktop",
        u"application/x-dia-diagram",
        u"application/x-fluid",
        u"application/x-glade",
        u"application/xhtml+xml",
        u"application/x-java-archive",
        u"application/x-m4",
        u"application/xml",
        u"application/x-object",
        u"application/x-perl",
        u"application/x-php",
        u"application/x-ruby",
        u"application/x-shellscript",
        u"application/x-sql",
        u"text/css",
        u"text/html",
        u"text/x-c",
        u"text/x-c++",
        u"text/x-chdr",
        u"text/x-copying",
        u"text/x-credits",
        u"text/x-csharp",
        u"text/x-c++src",
        u"text/x-csrc",
        u"text/x-dsrc",
        u"text/x-eiffel",
        u"text/x-gettext-translation",
        u"text/x-gettext-translation-template",
        u"text/x-haskell",
        u"text/x-idl",
        u"text/x-java",
        u"text/x-lisp",
        u"text/x-lua",
        u"text/x-makefile",
        u"text/x-objcsrc",
        u"text/x-ocaml",
        u"text/x-pascal",
        u"text/x-patch",
        u"text/x-python",
        u"text/x-sql",
        u"text/x-tcl",
        u"text/x-troff",
        u"text/x-vala",
        u"text/x-vhdl",
]

#ALL_MIMETYPES = DOCUMENT_MIMETYPES + IMAGE_MIMETYPES + AUDIO_MIMETYPES +\
#                VIDEO_MIMETYPES + DEVELOPMENT_MIMETYPES

FILTERS = {
        # dict of name as key and the matching mimetypes as value
        # if the value is None this filter matches all mimetypes
        "DOCUMENT": MimeTypeSet(*DOCUMENT_MIMETYPES),
        "IMAGE": MimeTypeSet(*IMAGE_MIMETYPES),
        "MUSIC_PIECE": MimeTypeSet(*AUDIO_MIMETYPES),
        "VIDEO": MimeTypeSet(*VIDEO_MIMETYPES),
        "SOURCE_CODE": MimeTypeSet(*DEVELOPMENT_MIMETYPES),
}

def get_interpretation_for_mimetype(mimetype):
    matching_filter = None
    i = 0
    for filter_name in FILTERS.keys():
        if mimetype and mimetype in FILTERS[filter_name]:
            print filter_name, "---", FILTERS[filter_name]
            matching_filter = filter_name
        i += 1
        print i 
    print "***", matching_filter
    if matching_filter:
        return getattr(Interpretation, matching_filter).uri
    return ""

